<?php

require_once 'lib/3rdparty/Michelf/MarkdownExtra.inc.php';
use Michelf\MarkdownExtra;
        
class GILSUtils {


    public static function parseLinkHeader($link){
    
    /*
    
    <http://gitlab.bentasker.co.uk/api/v4/projects?membership=false&order_by=created_at&owned=false&page=2&per_page=20&repository_checksum_failed=false&simple=false&sort=desc&starred=false&statistics=false&wiki_checksum_failed=false&with_custom_attributes=false&with_issues_enabled=false&with_merge_requests_enabled=false>; rel="next", <http://gitlab.bentasker.co.uk/api/v4/projects?membership=false&order_by=created_at&owned=false&page=1&per_page=20&repository_checksum_failed=false&simple=false&sort=desc&starred=false&statistics=false&wiki_checksum_failed=false&with_custom_attributes=false&with_issues_enabled=false&with_merge_requests_enabled=false>; rel="first", <http://gitlab.bentasker.co.uk/api/v4/projects?membership=false&order_by=created_at&owned=false&page=3&per_page=20&repository_checksum_failed=false&simple=false&sort=desc&starred=false&statistics=false&wiki_checksum_failed=false&with_custom_attributes=false&with_issues_enabled=false&with_merge_requests_enabled=false>; rel="last"
    
    */
    
        $parts = explode(",", $link);
        $x = 0;
        $vals = array();
        foreach ($parts as $part){
            $p = explode(";", $part);
//             print_r($p);
            $link = str_replace(array("<",">"),"",$p[0]);
            $rel = explode('"',$p[1])[1];
            $vals[$rel] = array(
                "link" => GILSUtils::forceHTTPS($link),
                "rel" => $rel
            );
            $x++;
        }
        //print_r($vals);
        return $vals;
    }

    public static function forceHTTPS($url){
        global $config;
        return ($config->force_https) ? str_replace("http://", "https://", $url) : $url;
    }
    
    
    public static function markdownToHTML($md){
        return MarkdownExtra::defaultTransform($md);
    }
    
    
    public static function qs2sef($qs, $ext=".html"){
            //echo $qs;
            $pairs = explode("&", $qs);
            
            $set = array();
            $url = array();
            foreach ($pairs as $pair){
                $ps = explode("=", $pair);
                $set[$ps[0]] = $ps[1];
            }
    
            if (isset($set['proj'])){
                $url[] = "project";
                $url[] = str_replace(" ","-",$set['proj']);
            }elseif(isset($set['issue'])){
                // issue={$issue['gitlab_reference']}
                $url[] = "issue";
                $url[] = str_replace("#", "/", $set['issue']);
            }elseif(isset($set['wiki'])){
                $url[] = "wiki";
                $url[] = $set['wiki'];
                if (isset($set['page'])){
                    $url[] = "page";
                    $url[] = $set['page']; # this is the slug
                }else{
                    $url[] = "list";
                }
            }elseif(isset($set['commit'])){
                    //commit={$commit}&proj={$pth}
                    $url[] = "commits";
                    $url[] = $set['commitproj'];
                    
                    if(isset($set['commit'])){
                        $url[] = $set['commit'];
                    }else{
                        $url[] = "list";
                    }
            }elseif(isset($set['container'])){
                    $url[] = "containers";
                    $url[] = $set['contproj'];
                    $url[] = $set['container'];
            }
    
    
            return "/".implode("/", $url).$ext;
    }
    
    public static function sef2qs($sef){
    
        $path_split = explode("/", str_replace(".html", "",$sef));
        //print_r($path_split);
        
        if (count($path_split) == 1 || $path_split[1] == "" || $path_split[1] == "index"){
            return array(
                    "section" => "projects"
            );
        }
        
        switch($path_split[1]){
            case "avatars":
                return array(
                    "section" => "avatars",
                    "path" => implode("/", array_slice($path_split, -2))
                );
                break;
            
            case "commits":
                return array(
                    "section" => "commits",
                    "subsection" => implode("/",array($path_split[2], $path_split[3])),
                    "commit" => end($path_split)
                    );
            
            case "containers":
                return array(
                    "section" => "containers",
                    "subsection" => implode("/",array($path_split[2], $path_split[3])),
                    "container" => end($path_split)
                    );            
            
            case "project":
                return array(
                    "section" => "project",
                    "subsection" => implode("/",array_slice($path_split, -2)),
                    "mode" => "list",
                    "sef" => $sef        
                );
                
            case "issue":
                if ($path_split[4] == "uploads"){
                    // It's an image request
                    return array(
                        "section" => "issue",
                        "subsection" => implode("/",array($path_split[2], $path_split[3])),
                        "issid" => false,
                        "img_path" => implode("/",array_slice($path_split, -3)),
                        "sef" => $sef                        
                    );
                }
            
                return array(
                    "section" => "issue",
                    "subsection" => implode("/",array_slice(array_slice($path_split, -3),0, 2)),
                    "issid" => end($path_split),
                    "sef" => $sef                        
                );
                
            case "wiki":
                $act = $path_split[4];
                $page = "";
                if ($act == "page"){
                    $page = $path_split[5];
                }
                return array(
                    "section" => "wiki",
                    "subsection" => implode("/", array($path_split[2], $path_split[3])),
                    "action" => $act,
                    "page" => $page
                );
        }
    }
    
    public static function translateUser($user){
        return $user;
    }
      
      
    public static function replaceCommitRefsEtc($md, $project_path){
        /** Detect commit hashes in a block of markdown and replace it with a link
        
         Implemented for websites/Gitlab-Issue-Listing-Script#8
        
        */
    
    
        $proj_path_chars="a-zA-Z_\-\/";
        // Commits
        // ((([^\ ]+@)?)([a-f,0-9]{40}))
        $matches = array();
        preg_match_all("((([$proj_path_chars]+@)?)([a-f,0-9]{40}))", $md, $matches);
        //print_r($matches);
        
        if (count($matches) > 0 && count($matches[0]) > 0){
            // iter over, calculate and perform the replacement
            foreach($matches[0] as $ref){
                $splt = explode("@", $ref);
                $pth = (count($splt) == 1) ? $project_path : $splt[0];
                $commit = (count($splt) == 1) ? $splt[0] : $splt[1];
                $sc = substr($commit, 0, 7);
               
                $newlink = GILSUtils::qs2sef("commit={$commit}&commitproj={$pth}");
               
                // Turn it into a link
                $newmd = "[$sc]($newlink)";
                $md = str_replace($ref, $newmd, $md);
            }
        
        }
        
        
        
        // Project references
        // (([a-zA-Z-\/]+)?)(#[0-9]+)
        $matches = array();
        preg_match_all("((([$proj_path_chars]+)?)(#[0-9]+))", $md, $matches);
        //print_r($matches);
        
        if (count($matches) > 0 && count($matches[0]) > 0){
            // iter over, calculate and perform the replacement
            foreach($matches[0] as $ref){
                //echo $ref;
                $splt = explode("#", $ref);
                $pth = (count($splt) == 1) ? $project_path : $splt[0];
                $issno = (count($splt) == 1) ? $splt[0] : $splt[1];              
                $nref="{$pth}#$issno";
                $link = GILSUtils::qs2sef("issue={$ref}");
               
                // Turn it into a link
                $newmd = "[$nref]($link)";
                $md = str_replace($ref, $newmd, $md);
            }
        
        }
        
        
        /*foreach ($matches as $match){
            print_r($match);
        }*/
    
        return $md;
    }
      
      
      
    public static function getAvatarUrl($url){
        if ($url == ""){
            return False;
        }
    
        $url_parts = explode("/", $url);
        $url_a=array('avatars');
        foreach(array_slice($url_parts, -2) as $a){
            $url_a[] = $a;
        }
        return "/". implode("/", $url_a);
    }
}
